;                                                                               
; Game concept for the Atari 2600 VCS
; 
; Copyright 2001
; By Joe Grand, jgrand@mindspring.com
; 
; dasm source.s -f3 -osource.bin to compile 
; 
; Tested with StellaX v1.1.3a
;
; TODO
; 
; 1. DONE! Draw score, logo, initial playfield [initial playfield]
; 2. DONE! Enable movement up and down with joystick [joystick jg-1]
; 3. DONE! Display drive head on the right side of the screen ...
; 4. DONE! Draw playfield, with moving drive head [drivehead jg-2]
; 

; 
; 5. Add data blobs 
; 6. Draw new logo/title banner for bottom of the screen
; 7. Generate sound when button is pressed
; 
; POTENTIAL GAME FEATURES
; 
; 6- or 4-digit score routine, with hexadecimal score? 
; Paddle instead of joystick  
; Data latency buffer    
; Bad blocks   
; Various size data (fragmented) 
; Colored data blocks (ala Simon) and data sequence to read per level?
; 

; 
; Clear memory and registers
; Set up variables
; Loop:
;    Do the vertical blank
;    Do game calculations
;    Draw screen
;    Do more calculations during overscan
;    Wait for next vertical blank
; End Loop.
; 

        processor 6502
        include vcs.h

        SEG.U vars
        org $80         ; Allocate memory, PIA has 128 bytes of RAM ($80-$FF). Stack from $FF

Temp                    ds      1
TempX1                  ds      1
TempY1                  ds      1
DriveHeadYPos           ds      1       ; Vertical position of drive head
DataIndex               ds      1       ; Counter for drawing 
Joystick0               ds      1       ; Player 1's joystick value
Counter                 ds      1       ; Counter for drawing score

ZColorScore             ds      1
ZColorDriveHead         ds      1
ZColorData              ds      1
ZColorLine              ds      1       ; Track seperator 
ZColorTrack             ds      1
ZColorBackground        ds      1

ScorePtr0               ds      2       ; Pointer to score shapes
ScorePtr1               ds      2 
ScorePtr2               ds      2 
ScorePtr3               ds      2 
ScorePtr4               ds      2 
ScorePtr5               ds      2 

CarMotions              ds      10      ; Motion values for each car
CarMotionTimers         ds      10      ; Timer before car is moved
ZCarPatterns            ds      10      ; Current size and multiples of cars

;GameNumber             ds      1       ; Current game variation being played (0-7)     
;CurrentLine            ds      1       ; Current line being drawn                      

; All variables from here to $FF are zero'd out every new game

Score                   ds      3       ; Player score (6 hex digits)

        SEG code
        org $F800       ; Limit ourselves to 2K like the old days

Start

;
; The 2600 powers up in a completely random state, except for the PC which
; is set to the location at $FFC.  Therefore the first thing we must do is
; to set everything up inside the 6502.
;

        SEI             ; Disable interrupts, if there are any.
        CLD             ; Clear BCD math bit.
        LDX  #$FF
        TXS             ; Set stack to beginning.

        ; Loop to clear memory and TIA registers
        
        LDA  #0
B1      STA  0,X
        DEX
        BNE  B1

        ; The above routine does not clear location 0, which is VSYNC

        JSR  GameInit           ; Initial variable and register setup
MainLoop
        JSR  VerticalBlank      ; Execute the vertical blank.
        JSR  CheckSwitches      ; Check console switches.
        JSR  GameCalc           ; Do calculations during VBLANK
        JSR  DrawScreen         ; Draw the screen
        JSR  OverScan           ; Do more calculations during overscan
        JMP  MainLoop           ; Continue forever.


VerticalBlank  ;*********************** VERTICAL BLANK HANDLER

;
; Beginning of the frame - at the end of overscan.
;

        LDA  #2         ; VBLANK was set at the beginning of overscan.
        STA  WSYNC  
        STA  WSYNC
        STA  WSYNC
        
        STA  VSYNC      ; Begin vertical sync
        
        STA  WSYNC      ; First line of VSYNC
        STA  WSYNC      ; Second line of VSYNC

        ; Set the timer to go off just before the end of vertical blank space
        ; 37 scanlines * 76 cycles = 2812 cycles / 64 = approx. 44
        
        LDA  #44
        STA  TIM64T
;       
; End the VSYNC period.
;       
        LDA  #0
        STA  WSYNC ; Third line of VSYNC.
        STA  VSYNC ; (0)
        RTS  


CheckSwitches ;*************************** CONSOLE SWITCH HANDLER

        ;LDA  #0         ; Clear collision latches
        ;STA  CXCLR      

        LDA  SWCHA      ; Read joystick 0
        STA  Joystick0  ; Store for later use


        LDA  SWCHB      ; Read the switches
        LSR             ; Shift the Reset switch
        ROR             ; Rotate Reset to Negative flag and Carry has select
        
        ; PAL/NTSC select check

        RTS

;
; Minimal game calculations, just to get the ball rolling.
;
GameCalc ;******************************* GAME CALCULATION ROUTINES

        LDA  #$30
        STA  CTRLPF             ;No Reflect,No Score,Low Priority,8 pixel-wide ball

        ;
        ; Set pointers to number data (based on current score)
        ; Score Score+1 Score+2
        ;
        LDX  #3-1
        LDY  #10
ScoreLoop
        LDA  Score,X
        AND  #$0F               ; Mask it
        ASL                     ; Multiply by 8
        ASL
        ASL                

        ADC  #<FontPage         ; Store the 2-byte pointer
        STA  ScorePtr0,Y
        LDA  #0
        ADC  #>FontPage
        STA  ScorePtr0+1,Y   
        
        DEY
        DEY
        
        LDA  Score,X
        AND  #$F0               ; Mask it
        LSR                     ; $00,$08,$10,$18,$20, ... ( / 8 = value)
                
        ADC  #<FontPage         ; Store the 2-byte pointer
        STA  ScorePtr0,Y
        LDA  #0
        ADC  #>FontPage
        STA  ScorePtr0+1,Y

        DEY
        DEY
        DEX
        BPL  ScoreLoop
        
        ;
        ; Set new vertical position of drive head based on joystick value
        ;
;        LDA  Joystick0          ; Load the joystick switches
;        ROL
;        ROL
;        BMI  NotDown            ; Joystick pushed down?
;        LDX  DriveHeadYPos
;        CPX  BottomYPos
;        BEQ  NoStick            ; If DriveHeadYPos < BottomYPos, don't change
;        DEC  DriveHeadYPos
;        DEC  DriveHeadYPos
;        JMP  NoStick
;NotDown
;        ROL
;        BMI  NoStick            ; Joystick pushed up?
;        LDX  DriveHeadYPos
;        CPX  TopYPos
;        BEQ  NoStick            ; If DriveHeadYPos > TopYPos, don't change
;        INC  DriveHeadYPos
;        INC  DriveHeadYPos
;NoStick
        RTS


DrawScreen ;**************************** SCREEN DRAWING ROUTINES
;
; Initialize display variables.
;
        LDA  ZColorBackground
        STA  COLUBK             ; Background will be black.
        LDA  ZColorScore        ; Set the score color (P0 and P1)
        STA  COLUP0
        STA  COLUP1
        LDX  #3
        STX  NUSIZ0             ; Three copies of player, close together
        STX  NUSIZ1

WaitVBlank
        LDA  INTIM              ; Loop until timer is done - wait for the proper scanline
                                ; (i.e. somewhere in the last line of VBLANK)
        BNE  WaitVBlank         ; Whew! We're at the beginning of the frame
        STA  WSYNC              ; End the current scanline
        STA  HMOVE              ; Add horizontal motion to position sprites
        STA  VBLANK             ; End the VBLANK period with a zero, enable video.
        STA  WSYNC              ; Wait 2 scanlines before drawing score
        STA  WSYNC

        LDX  #22
Pause1
        DEX
        BNE  Pause1
        STA  RESP0
        
        STA  WSYNC
        LDX  #22
Pause2
        DEX
        BNE  Pause2
        NOP
        STA  RESP1
        STA  HMCLR
        LDA  #%10100000         ; P1 position 2 less than P0
        STA  HMP1
        LDA  #%11000000
        STA  HMP0
        STA  WSYNC
        STA  HMOVE
        STA  WSYNC

      
ScanLoop ; ================================== SCANNING LOOP
;
; There are 228/3 cycles per line, so we have a maximum of 73 cycles 
; after STA WSYNC
;
; Display the scores - 6 digit, courtesy of Robin Harbron
;
         
        LDA  #8-1                       ; Lines to display
        STA  Counter
        
        LDX  #10
Pause3
        DEX
        BNE  Pause3
        NOP
        NOP
        NOP
        
DrawScore
        LDY  Counter            ; 3
        LDA  (ScorePtr0),Y      ; 5
        STA  GRP0               ; 3 = 11

        BIT  $00

        LDA  (ScorePtr1),Y      ; 5
        STA  GRP1               ; 3
        LDA  (ScorePtr5),Y      ; 5
        TAX                     ; 2
        TXS                     ; 2
        LDA  (ScorePtr2),Y      ; 5
        STA  Temp               ; 3
        LDA  (ScorePtr3),Y      ; 5
        TAX                     ; 2
        LDA  (ScorePtr4),Y      ; 5 = 37

        LDY  Temp               ; 3
        STY  GRP0               ; 3
        STX  GRP1               ; 3
        STA  GRP0               ; 3
        TSX                     ; 2
        STX  GRP1               ; 3 = 17   
        
        DEC  Counter            ; 5
        BPL  DrawScore          ; 3 =  8
         
        LDA  #0
        STA  GRP0
        STA  GRP1 
        
        ;
        ; Skip down a few lines
        ; Do other calculations here if necessary, we have lots of time
        ;        
        LDY  #6
DrawWait
        STA  WSYNC
        DEY
        BNE  DrawWait

        ; Start drawing the actual disk playfield (10 lanes, 15 lines each)
;        LDX  #0                 ; P0
;        LDA  #140
;        JSR  SetMotionRegsX     ; Set proper X axis for drivehead
;        STA  HMOVE              ; Add horizontal motion to position sprites
        ;NOP
        ;NOP
        ;NOP
        ;NOP
        ;STX  HMCLR             ; Clear horizontal motion registers
;        LDY  TopYPos            ; Set bounds on playfield
;        LDA  ZColorDriveHead    ; Setup colors
;        STA  COLUP0
;        LDA  ZColorData
;        STA  COLUP1
        
        ;STA  WSYNC             ; Wait for sync
;DrawTrack
        ;
        ; Is Y at the right position to draw drivehead?
        ;
;        CPY  DriveHeadYPos
;        BNE  NoYPos
;        LDX  #16-1              ; Drivehead index        
;NoYPos
;        CPX  #-1                ; If index < 0, graphics are finished drawing
;        BEQ  NoHead
        
        ;
        ; Time to draw a line of the drivehead
        ;
;        LDA  HeadFrame,X        ; Get the shape of the drivehead
;        STA  GRP0               ; Draw it
;        DEX
;NoHead
        ;
        ; Calculate data positions and load graphics
        ;
;        STX  TempX1             ; Store X to temp since it is used while drawing drivehead
;        TYA  
;        LSR                     ; Current track number = Y / 16
;        LSR 
;        LSR 
;        LSR 
;        STA  Score              ; Debug to show track number
        ;TAX    

        ;LDA  #CarMotions,X     ; Fine horizontal position
        ;LDA  #0
        ;STA  HMOVE
        ;STA  HMP1

        ;STY  TempY1
        ;LDX  #1                ; P1
        ;LDA  #40
        ;JSR  SetMotionRegsX    ; Set proper X axis for data
        ;STA  HMOVE             ; Add horizontal motion to position sprites
        ;
        ;Time to draw a line of the data for the track we're currently in
        ;
        ;LDY  TempY1
        ;LDX  DataIndex
        ;LDA  DataFrame,X       ; Get the shape of the data
        ;STA  GRP1              ; Draw it
        ;DEC  DataIndex
        ;BPL  NoReset
        ;LDX  #16-1
        ;STX  DataIndex         ; Reset index

;NoReset
;        STA  WSYNC              ; Wait for sync
;        LDX  TempX1             ; Restore X
;        TYA
;        AND  #15                ; Modulo 16 ( % 2^n = AND (2^n)-1 )
;        BNE  TrackColor
;        LDA  ZColorLine         ; Draw track seperator every 16th line
;        JMP  SetColor
;TrackColor
;        LDA  ZColorTrack
;SetColor
;        STA  COLUBK             ; Draw the track background, either seperator or not
;        DEY
;        BNE  DrawTrack
;
;        STA  WSYNC              ; End the current scan line
        
        ;
        ; Skip down a few lines
        ; Do other calculations here if necessary, we have lots of time
;        LDA  ZColorLine
;        STA  COLUBK
;        STY  GRP0               ; Clear graphics
;        STY  GRP1
;        STA  WSYNC              ; Draw the bottom border of the disk

        LDA  ZColorBackground   ; Black line
        STA  COLUBK
        LDY  #10
DrawWait2
        STA  WSYNC
        DEY
        BNE  DrawWait2

;
; Prepare to draw the logo
; Waste 1 scan line
;
        STA  HMOVE
        NOP
        NOP
        NOP
        NOP
        STY  HMCLR      ; Clear horizontal motion registers
        INY             ; X = 1
        STY  NUSIZ0     ; 2 shapes, close together
        STY  NUSIZ1
        STA  RESP0      ; Set the X coordinate
        STA  RESP1                   
        LDA  #$10       ; Move P1 one pixel to the left
        STA  HMP1
        LDA  ZColorData ; Is also the logo color
        STA  COLUP0
        STA  COLUP1

;
; Now draw the 8 lines
; Note: All the shapes end with a shape of zero. This way, I can assume
; the video display is pure black after the loop is finished.
;
        LDX  #8-1               ; 8 scan lines
DrawLogo
        STA  WSYNC              ; 3 Wait for sync
        STA  HMOVE              ; 3 Move the players (Only first time)
        LDA  Logo1,X            ; 4 Draw the shape
        STA  GRP0               ; 3
        LDA  Logo2,X            ; 4 Draw in player #1
        STA  GRP1               ; 3
        NOP                     ; 2 Waste 4 cycles
        NOP                     ; 2
        LDA  Logo4,X            ; 4 Preload the final 2 bytes
        TAY                     ; 2
        LDA  Logo3,X            ; 4
        STA  GRP0               ; 3 Store in shape (Cycle 34)
        STY  GRP1               ; 3
        STA  HMCLR              ; 3 Clear horiz. motion for future loops
        DEX                     ; 2 All done?
        BPL  DrawLogo           ; 3 Loop

;
; Clear all registers here to prevent any possible bleeding.
;
        LDA  #2
        STA  WSYNC      ; Finish this scanline.
        STA  VBLANK     ; Make TIA output invisible,
        ; Now we need to worry about it bleeding when we turn
        ; the TIA output back on.
        LDY  #0
        STY  PF0
        STY  PF1
        STY  PF1
        ;STY  GRP0
        ;STY  GRP1
        STY  ENAM0
        STY  ENAM1
        STY  ENABL
        RTS


OverScan   ;***************************** OVERSCAN CALCULATIONS
;
; For the Overscan routine, one might take the time to process such
; things as collisions.
;
        LDX  #30
KillLines
        STA  WSYNC
        DEX
        BNE  KillLines
        RTS


GameInit   ;***************************** INITIAL VARIABLE & REGISTER SETUP
;
; GameInit could conceivably be called when the Select key is pressed,
; or some other event.
;
        ;
        ;Load base colors 
        ;
        LDX  #6-1
NextColor
        LDA  BaseColors,X       ; Get the colors
        STA  ZColorScore,X      ; Save the adjusted colors
        ;CPX  #4
        ;BPL  NotHardwareYet
        ;STA  COLUP0,X          ; Set the hardware defaults (P0, P1, PF, BK)
NotHardwareYet
        DEX
        BPL  NextColor

        ;
        ; Clear data pointers
        ;
        LDX  #11
        LDA  #0
Clear2
        STA  ScorePtr0,X
        DEX
        BPL  Clear2

        STA  WSYNC
        LDY  #3
Pos0
        DEY
        BNE  Pos0
        STA  RESP0
        
        ;
        ; Initialize score value
        ;
        ;STA  Score      ; A = 0
        ;STA  Score+1
        ;STA  Score+2
        
        LDA  #$AB
        STA  Score
               
        LDA  #$CD
        STA  Score+1
         
        LDA  #$EE
        STA  Score+2
        
        ;
        ; TODO: Set data direction registers for paddles
        ;
        
        LDA  #88                ; Start with drivehead positioned in center
        STA  DriveHeadYPos    

        ;LDA  #16-1             ; Width of track
        ;STA  DataIndex

        ;
        ;Initialize data
        ;
        LDX  #10-1              ; 10 lanes
DataInit
        LDA  #1                 ; Allow motion in 2 frames
        STA  CarMotionTimers,X
        ;LDA  CarColors,X       ; Copy the car colors
        ;STA  ZCarColors,X
        ;CLC
        ;LDA  GameNumber        ; Get the traffic pattern
        ;AND  #3
        ;TAY
        ;TXA
        ;ADC  Mul10Tbl1,Y       ; Mul 0-3 by 10
        ;TAY
        ;LDA  CarPatterns,Y     ; Save the pattern
        ;STA  ZCarPatterns,X
        LDA  #$60               ; Horizontal motion (+6 pixels)
        STA  CarMotions,X
        DEX
        BPL  DataInit

        RTS


;
; Set the horizontal motion registers
; Kills Y
;
;SetMotionRegsX  
;        JSR  CalcXRegs  ; Calculate the values
;        STA  HMP0,X     ; Save the horizontal motion register
;        STA  WSYNC      ; Sync to video
;Wait    DEY             ; Wait for cycle delay (5 cycles per)
;        BPL  Wait
;        STA  RESP0,X    ; Hit the register for course adjustment
;        RTS


;
; Given a requested X coord in A,
; return in A and Y and Horizonal offset and the cycle delay
;
;CalcXRegs       
;        CLC
;        ADC  #46
;        TAY
;        AND  #$F        ; Mask the 4 bit offset
;        STA  TempX1     ; Save temp
;        TYA             ; Get value again
;        LSR             ; Isolate the upper 4 bits
;        LSR
;        LSR
;        LSR
;        TAY             ; Save as a WSYNC Delay
;        CLC
;        ADC  TempX1     ; Mod 15
;        CMP  #15
;        BMI  NoExcess
;        SBC  #15        ; Remove one step
;        INY             ; 5 more cycles
;NoExcess        
;        EOR  #7         ; Negate horizontal offset
;        ASL             ; Move to upper 4 bits (Hardware)
;Waste18 
;        ASL             ; 2 Call to waste 18 cycles
;        ASL             ; 2
;Waste14 
;        ASL             ; 2 Call to waste 14 cycles
;Waste12 
;        RTS             ; 6 Call to waste 12 cycles



;
; Logo bitmap, 32 x 8
;

Logo1     
                .byte   %00000000 ; |        |
                .byte   %11100111 ; |XXX  XXX|
                .byte   %00010100 ; |   X X  |
                .byte   %00010100 ; |   X X  |
                .byte   %00010100 ; |   X X  |
                .byte   %11110100 ; |XXXX X  |
                .byte   %10000100 ; |X    X  |
                .byte   %11110111 ; |XXXX XXX|

Logo2     
                .byte   %00000000 ; |        |
                .byte   %01110010 ; | XXX  X |
                .byte   %00001010 ; |    X X |
                .byte   %00001010 ; |    X X |
                .byte   %00001010 ; |    X X |
                .byte   %01111000 ; | XXXX   |
                .byte   %01000010 ; | X    X |
                .byte   %01111010 ; | XXXX X |

Logo3     
                .byte   %00000000 ; |        |
                .byte   %11100101 ; |XXX  X X|
                .byte   %00010101 ; |   X X X|
                .byte   %00010101 ; |   X X X|
                .byte   %00010101 ; |   X X X|
                .byte   %11110001 ; |XXXX   X|
                .byte   %10000101 ; |X    X X|
                .byte   %11110101 ; |XXXX X X|

Logo4     
                .byte   %00000000 ; |        |
                .byte   %11001111 ; |XX  XXXX|
                .byte   %01101000 ; | XX X   |
                .byte   %00101000 ; |  X X   |
                .byte   %00101000 ; |  X X   |
                .byte   %00101111 ; |  X XXXX|
                .byte   %01101000 ; | XX X   |
                .byte   %11001111 ; |XX  XXXX|

HeadFrame       
                .byte   %00000000 ; |        |
                .byte   %00011110 ; |   XXXX |
                .byte   %00011110 ; |   XXXX |
                .byte   %00110011 ; |  XX  XX|
                .byte   %00110001 ; |  XX   X|
                .byte   %00111000 ; |  XXX   |
                .byte   %01001100 ; | X  XX  |
                .byte   %11111100 ; |XXXXXX  |
                .byte   %01001100 ; | X  XX  |
                .byte   %00111000 ; |  XXX   |
                .byte   %00110001 ; |  XX   X|
                .byte   %00110011 ; |  XX  XX|
                .byte   %00011110 ; |   XXXX |
                .byte   %00011110 ; |   XXXX |
                .byte   %00000000 ; |        |
                .byte   %00000000 ; |        |

DataFrame       
                .byte   %00000000 ; |        |
                .byte   %00000000 ; |        |
                .byte   %01100110 ; | XX  XX |
                .byte   %11111110 ; |XXXXXXX |
                .byte   %11001111 ; |XX  XXXX|
                .byte   %10110011 ; |X XX  XX|
                .byte   %10110011 ; |X XX  XX|
                .byte   %10110011 ; |X XX  XX|
                .byte   %10110011 ; |X XX  XX|
                .byte   %10110011 ; |X XX  XX|
                .byte   %10110011 ; |X XX  XX|
                .byte   %10110011 ; |X XX  XX|
                .byte   %11001111 ; |XX  XXXX|
                .byte   %11111110 ; |XXXXXXX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00000000 ; |        |

;
; Size and shapes for each traffic pattern for the 8 games
;
CarPatterns     
                HEX     50403020101020304050    ; Game #1,#5
                HEX     40312213041514233241    ; Game #2,#6
                HEX     46362016050016203646    ; Game #3,#7
                HEX     05152515050515251505    ; Game #4,#8

;CarColors      HEX     1AD8448824824A12DC42    ; Colors for each lane of cars


BaseColors      
                .byte   %10111000 ; Score/Time Remaining color (Green)
                .byte   %00011110 ; Drive Head color (Yellow)
                .byte   %00110100 ; Data color (Red)
                .byte   %11110100 ; Line color (Light Brown)
                .byte   %11110010 ; Track color (Brown)
                .byte   %00000000 ; Background color (Black)

TopYPos         
                .byte   160     ; Top-most scanline of playfield

BottomYPos      
                .byte   14      ; Bottom-most scanline of playfield


;
; Must not cross page boundaries!!
; All shapes are upside down to allow decrementing by Y as both
; a counter and a shape index
;
        org $FE00 ; *********************** GRAPHICS DATA

;
; Numeric font
;
FontPage
Zero            
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

One             
                .byte   %00111100 ; |  XXXX  |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00111000 ; |  XXX   |
                .byte   %00011000 ; |   XX   |

Two             
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %00111100 ; |  XXXX  |
                .byte   %00000110 ; |     XX |
                .byte   %00000110 ; |     XX |
                .byte   %01000110 ; | X   XX |
                .byte   %00111100 ; |  XXXX  |

Three           
                .byte   %00111100 ; |  XXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %00000110 ; |     XX |
                .byte   %01000110 ; | X   XX |
                .byte   %00111100 ; |  XXXX  |

Four            
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01001100 ; | X  XX  |
                .byte   %00101100 ; |  X XX  |
                .byte   %00011100 ; |   XXX  |
                .byte   %00001100 ; |    XX  |

Five            
                .byte   %01111100 ; | XXXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00000110 ; |     XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |

Six             
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100000 ; | XX     |
                .byte   %01100010 ; | XX   X |
                .byte   %00111100 ; |  XXXX  |

Seven           
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00001100 ; |    XX  |
                .byte   %00000110 ; |     XX |
                .byte   %01000010 ; | X    X |
                .byte   %01111110 ; | XXXXXX |

Eight           
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

Nine            
                .byte   %00111100 ; |  XXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00111110 ; |  XXXXX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |
                
A           
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |
                
B           
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |

C           
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |
                
D           
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |
                
E           
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |
                
F           
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |     


        .byte "J. Grand"

        org $FFFC
        
        .word Start
        .word Start
